require 'spec_helper'

module SpAllocationSchedulerTest
  describe 'SpAllocationScheduler createDefaultSpRecommendation' do
    include_context 'request metadata'
    date = Time.new
    current_year = date.year
    current_month = date.month
    past_month = date.month - 1

    before(:each) {
      past_month =  get_month(current_year, past_month)
      current_month = get_month(current_year, current_month)
    }

    def get_month(current_year, month)
      past_month_year = current_year
      if month == 0 # Handle case where current month = January
        month = 12
        past_month_year = current_year - 1
      end
      if month < 10 # handle 2 digit vs 1 digit case
        "#{past_month_year}-0#{month}"
      else
        "#{past_month_year}-#{month}"
      end
    end

    # smoke test that can run in each branch build, will just run DsaGeneration job
    # intended as basic verification of default quote request/response, and schema
    # only required if current month test cannot be run on branch builds, skip otherwise
    xit 'Run createDefaultSpRecommendation for previous month (smoke test)' do
      wrap_request do
        variables = {
          superartifactInfo: {
            accountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
            customerId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:customer_id] : TEST_CUSTOMER[:customer_id],
            month: past_month,
          }
        }
        graphql url(service: service),
          format_request(
            :createDefaultSpRecommendation,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:createDefaultSpRecommendation, service: service)
      end
      job_id = json_body[:data][:createDefaultSpRecommendation][:id]
      # this type of job usually takes < 60 secs, allowing 3 mins for max padding
      job_completion_check('SpDefaultRecommendation', job_id, 19)

      wrap_request do
        variables = {}
        graphql url(service: service),
          format_request(
            :node,
            service: service,
            crn: job_id,
            type: 'SpDefaultRecommendation',
            fields: 'id, status,  completedJobs, createdAt, finishedAt'
          ),
          variables, headers
        expect_success_graphql
        check(:node, service: service)
      end
      node_quote = json_body[:data][:node]
      expect(node_quote[:status]).to eq('COMPLETED')
      expect(node_quote[:completedJobs]).to include("DsaGeneration_#{past_month}")
      expect(node_quote[:completedJobs]).not_to include('30days_partialup_3yr__default_STANDARD')

      wrap_request do
        variables = {}
        graphql url(service: sp_api_service),
          format_request(
            :defaultSavingsPlanRecommendations,
            service: sp_api_service
          ),
          variables, headers
        expect_success_graphql
        check(:defaultSavingsPlanRecommendations, service: sp_api_service)
      end
      recommendations_array = json_body[:data][:defaultSavingsPlanRecommendations][:recommendations][:edges]
      verify_default_recommendations(recommendations_array)
    end

    it 'Run createDefaultSpRecommendation for current month (full default job creation)' do
      wrap_request do
        variables = {
          customOptions: {
            force: true
          },
          superartifactInfo: {
            accountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
            customerId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:customer_id] : TEST_CUSTOMER[:customer_id],
            month: current_month,
          }
        }
        graphql url(service: service),
          format_request(
            :createDefaultSpRecommendation,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:createDefaultSpRecommendation, service: service)
      end
      job_id = json_body[:data][:createDefaultSpRecommendation][:id]
      # allow 10 minutes for default quote creation to complete
      job_completion_check('SpDefaultRecommendation', job_id, 60)

      wrap_request do
        variables = {}
        graphql url(service: service),
          format_request(
            :node,
            service: service,
            crn: job_id,
            type: 'SpDefaultRecommendation',
            fields: 'id, status,  completedJobs, createdAt, finishedAt'
          ),
          variables, headers
        expect_success_graphql
        check(:node, service: service)
      end
      node_quote = json_body[:data][:node]
      expect(node_quote[:status]).to eq('COMPLETED')
      expect(node_quote[:completedJobs]).to include("DsaGeneration_#{current_month}")
      expect(node_quote[:completedJobs]).to include('7days_noup_3yr__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('7days_noup_3yr__default_STANDARD')
      expect(node_quote[:completedJobs]).to include('7days_partialup_3yr__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('7days_partialup_3yr__default_STANDARD')
      expect(node_quote[:completedJobs]).to include('7days_allup_3yr__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('7days_allup_3yr__default_STANDARD')
      expect(node_quote[:completedJobs]).to include('7days_noup_1yr__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('7days_noup_1yr__default_STANDARD')
      expect(node_quote[:completedJobs]).to include('7days_allup_1yr__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('7days_allup_1yr__default_STANDARD')
      expect(node_quote[:completedJobs]).to include('7days_partialup_1yr__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('7days_partialup_1yr__default_STANDARD')
      expect(node_quote[:completedJobs]).to include('30days_noup_3yr__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('30days_noup_3yr__default_STANDARD')
      expect(node_quote[:completedJobs]).to include('30days_partialup_3yr__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('30days_partialup_3yr__default_STANDARD')
      expect(node_quote[:completedJobs]).to include('30days_noup_1yr__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('30days_noup_1yr__default_STANDARD')
      expect(node_quote[:completedJobs]).to include('30days_allup_3yr__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('30days_allup_3yr__default_STANDARD')
      expect(node_quote[:completedJobs]).to include('30days_allup_1yr__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('30days_allup_1yr__default_STANDARD')
      expect(node_quote[:completedJobs]).to include('30days_partialup_1yr__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('30days_partialup_1yr__default_STANDARD')
      expect(node_quote[:completedJobs]).to include('SpAllocation__purchasedate30_STANDARD')
      expect(node_quote[:completedJobs]).to include('SpAllocation__default_STANDARD')
      expect(node_quote[:completedJobs]).to include("RiAllocation__purchasedate30_STANDARD_#{current_month}:#{past_month}")
      expect(node_quote[:completedJobs]).to include("RiAllocation__default_STANDARD_#{current_month}:#{past_month}")

      wrap_request do
        variables = {}
        graphql url(service: sp_api_service),
          format_request(
            :defaultSavingsPlanRecommendations,
            service: sp_api_service
          ),
          variables, headers
        expect_success_graphql
        check(:defaultSavingsPlanRecommendations, service: sp_api_service)
      end

      recommendations_array = json_body[:data][:defaultSavingsPlanRecommendations][:recommendations][:edges]
      verify_default_recommendations(recommendations_array)

      # smoke test drilldowns post quote creation by making a single simulation request for one default recommendation
      wrap_request do
        simulation_options = {
          options: {
            awsBillingAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
            considerExpiringUsage: false,
            evaluationPeriod: 'THIRTY_DAYS',
            hourlyCommitment: 3.0,
            offeringType: 'PARTIAL_UPFRONT',
            term: 'ONE_YEAR',
            type: 'COMPUTE'
          }
        }
        graphql url(service: sp_api_service),
          format_request(
            :savingsPlanSimulation,
            service: sp_api_service
          ),
          simulation_options, headers
        expect_success_graphql
        check(:savingsPlanSimulation, service: sp_api_service)
      end
      simulation_response = json_body[:data][:savingsPlanSimulation]
      on_demand_cost_covered_sum =
          simulation_response[:costData][:onDemandCostCovered].inject(0) {|sum, i|  sum + i }
      expect(on_demand_cost_covered_sum).to be > 0
      existing_on_demand_cost_covered_sum =
          simulation_response[:costData][:existingOnDemandCost].inject(0) {|sum, i|  sum + i }
      expect(existing_on_demand_cost_covered_sum).to be > 0
      expect(simulation_response[:costData][:existingOnDemandCost].length).to be > 0
      expect(simulation_response[:usageData][:edges].length).to be > 0
      expect(simulation_response[:summary][:coverageBefore]).to be >= 0
      expect(simulation_response[:summary][:coverageAfter]).to be >= simulation_response[:summary][:coverageBefore]
      expect(simulation_response[:summary][:effectiveDiscount]).to be >= 0
      expect(simulation_response[:summary][:globalDiscountRateBefore]).to be >= 0
      expect(simulation_response[:summary][:globalDiscountRateAfter]).to be >= simulation_response[:summary][:globalDiscountRateBefore]
      expect(simulation_response[:summary][:monthlySpend]).to be > 0
      expect(simulation_response[:summary][:lifetimeSavings]).to be > 0
      expect(simulation_response[:summary][:monthlySavings]).to be_within(1).of(simulation_response[:summary][:lifetimeSavings] * 729.6 / 8760)
      expect(simulation_response[:summary][:totalUpfrontCost]).to be_within(1).of((simulation_response[:hourlyCommitment] * 8760) /2)
    end

    def verify_default_recommendations(recommendations_array)
      expect(recommendations_array.length).to eq(6)
      recommendations_array.each {|recommendation|
        summary_node = recommendation[:node][:savingsPlanRecommendationSummary]
        num_of_years = get_sp_term_as_int(recommendation[:node][:savingsPlanConfiguration][:term])
        offering_type = recommendation[:node][:savingsPlanConfiguration][:offeringType]
        expect( recommendation[:node][:recommendedHourlyCommitment]).not_to be_nil
        annual_cost = recommendation[:node][:recommendedHourlyCommitment] * 8760

        if offering_type == 'ALL_UPFRONT'
          expected_total_cost = num_of_years * annual_cost
        elsif  offering_type == 'PARTIAL_UPFRONT'
          expected_total_cost = ((annual_cost / 2) * num_of_years).ceil
        else
          expected_total_cost = 0
        end

        expect(summary_node[:totalUpfrontCost]).not_to be_nil
        expect(summary_node[:totalUpfrontCost]).to be >= 0
        expect(summary_node[:totalUpfrontCost].round()).to eq(expected_total_cost.round()),
          "incorrect totalUpfrontCost, expected: #{expected_total_cost}," +
             "but got: #{summary_node[:totalUpfrontCost]}, for: #{recommendation}"

        if summary_node[:monthlySavings] > 0
          expected_monthly_savings = summary_node[:lifetimeSavings] * 729.6 / (num_of_years * 8760)
          tolerance_factor = expected_monthly_savings * 0.01
          expect(summary_node[:monthlySavings]).to be_within(tolerance_factor).of(expected_monthly_savings),
            "incorrect monthlySavings, expected: #{summary_node[:monthlySavings]}," +
               "to be within 1% of: #{expected_monthly_savings}, for: #{recommendation}"
        end

        expect(summary_node[:coverageBefore]).to be >= 0
        expect(summary_node[:coverageAfter]).to be > summary_node[:coverageBefore],
           "coverageAfter < coverageBefore for recommendation: #{recommendation}"

        if summary_node[:monthlySavings] > 0
          expect(summary_node[:globalDiscountRateBefore]).to be >= 0
          expect(summary_node[:globalDiscountRateAfter]).to be > summary_node[:globalDiscountRateBefore],
            "globalDiscountRateAfter < globalDiscountRateBefore for recommendation: #{recommendation}"
        end
      }
    end
  end
end
