require 'spec_helper'

module SpAllocationSchedulerTest
  describe 'Positive tests: SpAllocationScheduler createSpRecommendation' do
    include_context 'request metadata'
    future_date = Date.today + 60

    context 'create quote: 30 days with future purchase date, 3 year, all upfront' do
      variables = {
        customerInfo: CUSTOMER_INFO,
        input: {
          accountIds: [TEST_CUSTOMER[:owner_id]],
          evaluationPeriod: 'THIRTY_DAYS',
          paymentOption: 'ALL_UPFRONT',
          term: 'THREE_YEAR',
          purchaseDate: "#{future_date}T00:00:00Z"
        }
      }
      include_examples 'create and verify new recommendation', variables
    end

    context 'create quote: 30 days without future purchase date, 1 year, partial upfront' do
      variables = {
        customerInfo: CUSTOMER_INFO,
        input: {
          accountIds: [TEST_CUSTOMER[:owner_id]],
          evaluationPeriod: 'THIRTY_DAYS',
          paymentOption: 'PARTIAL_UPFRONT',
          term: 'ONE_YEAR'
        }
      }
      include_examples 'create and verify new recommendation', variables
    end

    context 'create quote: Seven days with future purchase date, 3 year, no upfront' do
      variables = {
        customerInfo: CUSTOMER_INFO,
        input: {
          accountIds: [TEST_CUSTOMER[:owner_id]],
          evaluationPeriod: 'SEVEN_DAYS',
          paymentOption: 'NO_UPFRONT',
          term: 'THREE_YEAR',
          purchaseDate: "#{future_date}T00:00:00Z"
        }
      }
      include_examples 'create and verify new recommendation', variables
    end

    context 'create quote: Seven days without future purchase date, 3 year, all upfront' do
      variables = {
        customerInfo: CUSTOMER_INFO,
        input: {
          accountIds: [TEST_CUSTOMER[:owner_id]],
          evaluationPeriod: 'SEVEN_DAYS',
          paymentOption: 'ALL_UPFRONT',
          term: 'ONE_YEAR'
        }
      }
      include_examples 'create and verify new recommendation', variables
    end

    context 'create quote: linked account, no data' do
      variables = {
        customerInfo: CUSTOMER_INFO,
        input: {
          accountIds: [TEST_CUSTOMER_LINKED_ACCOUNT[:owner_id]],
          evaluationPeriod: 'THIRTY_DAYS',
          paymentOption: 'ALL_UPFRONT',
          term: 'THREE_YEAR',
          purchaseDate: "#{future_date}T00:00:00Z"
        }
      }
      include_examples 'create and verify new recommendation', variables
    end

    context 'Waiting state: consolidated account' do
      variables = {
        customerInfo: CUSTOMER_INFO,
        input: {
          accountIds: [TEST_CUSTOMER[:owner_id]],
          evaluationPeriod: 'THIRTY_DAYS',
          paymentOption: 'ALL_UPFRONT',
          term: 'THREE_YEAR'
        }
      }
      include_examples 'Waiting state job verification', variables
    end

    # linked account used below will CURRENTLY produce NO_DATA quotes
    # marking below scenario as probable NO_DATA, as that's not
    # gauranteed to always be the case
    context 'Waiting state: linked account (probable NO_DATA)' do
      variables = {
        customerInfo: CUSTOMER_INFO,
        input: {
          accountIds: [TEST_CUSTOMER_LINKED_ACCOUNT[:owner_id]],
          evaluationPeriod: 'THIRTY_DAYS',
          paymentOption: 'ALL_UPFRONT',
          term: 'THREE_YEAR'
        }
      }
      include_examples 'Waiting state job verification', variables
    end
  end

  describe 'Negative tests: SpAllocationScheduler createSpRecommendation route' do
    include_context 'request metadata'

    xit 'route returns expected error for no auth request' do
      wrap_request do
        variables = {
          customerInfo: CUSTOMER_INFO,
          input: {
            accountIds: [
              accountIds: [TEST_CUSTOMER_LINKED_ACCOUNT[:owner_id]],
            ],
            evaluationPeriod: 'THIRTY_DAYS',
            paymentOption: 'ALL_UPFRONT',
            term: 'THREE_YEAR',
          }
        }
        graphql url(service: service),
          format_request(
            :createSpRecommendation,
            service: service
          ),
          variables, apitest_headers
        expect_status(401)
      end
    end

    it 'route returns expected error when customerInfo is empty' do
      wrap_request do
        variables = {
          customerInfo: {},
          input: {
            accountIds: [TEST_CUSTOMER_LINKED_ACCOUNT[:owner_id]],
            evaluationPeriod: 'THIRTY_DAYS',
            paymentOption: 'ALL_UPFRONT',
            term: 'THREE_YEAR',
          }
        }
        graphql url(service: service),
          format_request(
            :createSpRecommendation,
            service: service
          ),
          variables, headers
        expect_status(400)
      end
    end

    it 'route returns expected error when input is empty' do
      wrap_request do
        variables = {
          customerInfo: CUSTOMER_INFO,
          input: {}
        }
        graphql url(service: service),
          format_request(
            :createSpRecommendation,
            service: service
          ),
          variables, headers
        expect_status(400)
      end
    end

    it 'route returns expected error when accountId is a non-string' do
      wrap_request do
        variables = {
          customerInfo: CUSTOMER_INFO,
          input: {
              accountIds: [12345],
              evaluationPeriod: 'THIRTY_DAYS',
              paymentOption: 'ALL_UPFRONT',
              term: 'THREE_YEAR',
          }
        }
        graphql url(service: service),
          format_request(
            :createSpRecommendation,
            service: service
          ),
          variables, headers
        expect_status(400)
      end
    end

    it 'route returns expected error when invalid future date included' do
      wrap_request do
        variables = {
          customerInfo: CUSTOMER_INFO,
          input: {
            accountIds: [
              accountIds: [TEST_CUSTOMER_LINKED_ACCOUNT[:owner_id]],
            ],
            evaluationPeriod: 'THIRTY_DAYS',
            paymentOption: 'ALL_UPFRONT',
            term: 'THREE_YEAR',
            purchaseDate: "February 25th 2025"
          }
        }
        graphql url(service: service),
          format_request(
            :createSpRecommendation,
            service: service
          ),
          variables, headers
        expect(json_body[:errors]).not_to be_nil
        expect(json_body[:data]).to be_nil
      end
    end
  end
end
